<?php
namespace Controleur;

use Gateway\Connection;
use Gateway\UserGateway;
use Model\UserModel;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;

Class VisitorControler {

    private Connection $co;
    private UserGateway $gw;

    private UserModel $mdl;

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function __construct(Connection $co, string $action) {
        $this->co = $co;
        $this->gw = new UserGateway($this->co);
        $this->mdl = new UserModel($this->gw);
        $this -> getAction($action);
    }

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function getAction(string $action): void
    {
        switch ($action) {
            case "login":
                $this -> visitorLogIn();
                break;
            case "signin":
                $this -> visitorSignIn();
                break;
            default:
                break;
        }
    }

    public function accueil(){
        global $vues;
        require_once $vues['accueil'];
    }

    public function quote(array $arg){
        global $vues;
        $id=$arg['idQuote'] ?? 1;

        //echo "{$id}";
        require_once $vues['quote'];
    }

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function visitorLogIn(): void
    {
        
        global $vues;
        global $twig;
        echo $twig->render("login.html.twig");

        $this -> toLogIn();
    }

    public function toLogIn() : void
    {
        if ($_POST)
        {
            $pseudo = $_POST['pseudo'] ?? null;
            $mdp = $_POST['mdp'] ?? null;

            $user = $this -> mdl -> getUsername($pseudo);

            if ($user)
            {
                if (password_verify($mdp, $user->getPassword()))
                {
                    $_SESSION['pseudo'] = $pseudo;
                    $_SESSION['email'] = $user->getEmail();
                    header("Location: /");
                }
                else die(); /* Afficher Erreur Mot de passe */
            }
            else die(); /* Afficher Utilisateur non trouvé */
        }

    }

    /**
     * @throws SyntaxError
     * @throws RuntimeError
     * @throws LoaderError
     */
    public function visitorSignIn(): void
    {
        global $vues;
        global $twig;
        echo $twig->render("signin.html.twig");

        $this -> toSignIn();
    }


    public function toSignIn() : void
    {
        if ($_POST) {
            $pseudo = $_POST['pseudo'] ?? null;
            $email = $_POST['email'] ?? null;
            $mdp = $_POST['mdp'] ?? null;
            $cmdp = $_POST['cmdp'] ?? null;

            if ($mdp != $cmdp) {
                die(); /* Afficher Erreur Mot de passe */
            }

            $option = ['cost' => 12];
            $hmdp = password_hash($mdp, PASSWORD_BCRYPT, $option);

            $isUserAlreadyUsed = $this -> mdl -> getUsername($pseudo);
            $isEmailAlreadyUsed = $this -> mdl -> getEmail($email);

            if ($isUserAlreadyUsed || $isEmailAlreadyUsed)
                die(); /* Afficher Message Erreur */
            else echo $this->mdl->insertUser($pseudo, $email, $hmdp);

            $_SESSION["pseudo"] = $pseudo;
            $_SESSION["email"] = $email;

            header("Location: /");
        }
    }
}


